(ns poly.moving
(:use
  protege.core
  fit.points)
(:import 
  ru.igis.omtab.OpenMapTab
  com.bbn.openmap.omGraphics.OMGraphic
  com.bbn.openmap.proj.GreatCircle
  com.bbn.openmap.proj.ProjMath
  ru.igis.omtab.OMTPoly)
(:gen-class))
(def ^:dynamic *runa* ; RuNA
(OpenMapTab/getRuNA))
(def ^:dynamic *sta-points-attr* "STAPOINTS")
(def ^:dynamic *fin-points-attr* "FINPOINTS")
(def ^:dynamic *period-attr* "PERIOD")
(def ^:dynamic *finish-attr* "FINISH")
(defn put-attrs [obj lla1 lla2]
(.putAttribute obj *sta-points-attr* lla1)
(.putAttribute obj *fin-points-attr* lla2))
(defn reppp [lla le1 le2]
(apply concat (repeat-points (partition 2 lla) (points-repeater (/ le1 2)(/ le2 2)))))
(defn to-sec [tim]
(let [[n u] (re-seq #"\w+" tim)
      k (Integer. n)]
  (condp = u
     "min" (* 60 k)
     "hrs" (* 3600 k)
     "days" (* 24 3600 k)
     k)))
(defn find-mo [inst]
(first (filter #(= (.getName %) (sv inst "label")) (.. *runa* (getMapObs) (getTargets)))))
(defn pitos [itos]
; Parse instance toString and return internal instance name
(aget (.split (.substring itos 15) " ") 0))
(defn poly-moving-start [object observer time sec]
(try
  (if-let [obj (find-mo object)]
    (let [llaj (.getLLPoints obj)
           llas (map #(ProjMath/degToRad %) (OMTPoly/getLLPoints (svs observer "points")))
           lenj (alength llaj)
           lens (count llas)
           peri (to-sec time)
           finis (+ sec peri)]
       (.putAttribute obj *period-attr* peri)
       (.putAttribute obj *finish-attr* finis)
       (cond
         (> lenj lens) (put-attrs obj (vec llaj) (reppp llas lens lenj))
         (< lenj lens) (put-attrs obj (reppp llaj lenj lens) llas)
         (= lenj lens) (put-attrs obj (vec llaj) llas))
      1)
    -1)
(catch Exception ex
  (println ex)
  -1)))
(defn step-poly [ratio llas llaf]
(if (and (seq llas) (seq llaf))
  (let [[p1 l1 & llas2] llas
         [p2 l2 & llaf2] llaf
         d0 (GreatCircle/spherical_distance p1 l1 p2 l2)
         dis (* d0 ratio)
         azi (GreatCircle/spherical_azimuth p1 l1 p2 l2)
         poi (GreatCircle/spherical_between p1 l1 dis azi 1)]
    (lazy-cat [(aget poi 2)] [(aget poi 3)] (step-poly ratio llas2 llaf2)) )))
(defn poly-moving-repeat [object sec]
(try
  (if-let [obj (find-mo object)]
    (let [finis (.getAttribute obj *finish-attr*)]
      (if (>= sec finis)
          0
          (let [llas (.getAttribute obj *sta-points-attr*)
                 llaf (.getAttribute obj *fin-points-attr*)
                 peri (.getAttribute obj *period-attr*)
                 rest (- finis sec)
                 ratio (/ (- peri rest) peri)
                 ipp (step-poly ratio llas llaf)
                 arr (double-array ipp)]
             (.. obj (getLocationMarker) (setLocation arr (OMGraphic/RADIANS)))
             1)))
    -1)
(catch Exception ex
  (println ex)
  -1)))
(defn div-time [time len]
(let [sec (to-sec time)
        div (/ sec len)]
  (str div " sec")))
(defn poly-moving-start-n [mapob time sec]
(let [mm (into [] mapob)
       len (count mm)]
  (if (> len 1)
      (do (.addMapOb *runa* (first mm))
            (poly-moving-start (first mm)(second mm)(div-time time len) sec))
      -1)))
(defn poly-moving-repeat-n [mapob time N sec]
(let [mm (into [] mapob)
       len (count mm)
       fst (first mm)
       nxt (inc N)
       rep (poly-moving-repeat fst sec)]
   (if (and (= rep 0)(< nxt len))
       (poly-moving-start fst (nth mm nxt)(div-time time len) sec))
   rep))
