/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * Copyright (C) 2007 Ruslan P. Sorokin (sorokin@oogis.ru)
 * and Object-Oriented Geo-Information System Research Laboratory (http://www.oogis.ru)
 * All Rights Reserved.
 */
package ru.oogis.dro;

import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;
import java.util.StringTokenizer;

import edu.stanford.smi.protege.model.Instance;

public class CreatedEmitter extends DrlEmitter {

	public CreatedEmitter(Instance packInst) {
		super(packInst);
	}
	
	@SuppressWarnings("unchecked")
	public void createAndEmitRules(Writer wr) throws IOException{
		emitPrologue(wr);
		Collection swts = packInst.getOwnSlotValues(KB.s_suffix_wslot_tslot);
		String[] suffixes = new String[swts.size()];
		String[] wslots = new String[swts.size()];
		String[] tslots = new String[swts.size()];
		int i = 0;
		for (Iterator iter = swts.iterator(); iter.hasNext();) {
			String swt = (String) iter.next();
			StringTokenizer st = new StringTokenizer(swt);
			suffixes[i] = st.nextToken();
			wslots[i] = st.nextToken();
			tslots[i] = st.nextToken();
			i++;
		}
		Collection fcs = packInst.getOwnSlotValues(KB.s_for_creation);
		for (Iterator iter = fcs.iterator(); iter.hasNext();) {
			Instance fc = (Instance) iter.next();
			for (int j = 0; j < tslots.length; j++) {
				emitCreatedRule(fc, wr, suffixes[j], wslots[j], tslots[j]);
			}
		}
		Collection rls = packInst.getOwnSlotValues(KB.s_rules);
		for (Iterator iter = rls.iterator(); iter.hasNext();) {
			Instance rule = (Instance) iter.next();
			emitRule(rule, wr);
		}
		wr.close();
	}

	public void createAndEmitRules(String path) throws IOException{
		createAndEmitRules(new FileWriter(path));
	}
	
	public void emitCreatedRule(Instance rule, Writer wr, String suffix, String wslot, String tslot) throws IOException{
		Object rnm = rule.getOwnSlotValue(KB.s_title);
		if(rnm == null)
			rnm = rule.getOwnSlotValue(KB.s_name);
		Object dia = rule.getOwnSlotValue(KB.s_dialect);
		if(dia == null)
			dia = "java";
		Object whn = rule.getOwnSlotValue(KB.kb.getSlot(wslot));
		Object thn = rule.getOwnSlotValue(KB.kb.getSlot(tslot));
		if(rnm != null && whn != null && thn != null){
			wr.write(DrlParser.RULE+" \""+rnm+"-"+suffix+"\"\n\t");
			wr.write(DrlParser.DIALECT+" \""+dia+"\"\n\t");
			wr.write(DrlParser.WHEN+"\n"+whn+"\n\t");
			wr.write(DrlParser.THEN+"\n"+thn+"\n");
			wr.write(DrlParser.END+"\n\n");
		}
	}
	
	public void run(){
		try {
			createAndEmitRules(pipw);
		} catch (IOException e) {
			e.printStackTrace();
		}
	}

	@SuppressWarnings("unchecked")
	public static void main(String[] args) throws Exception {
		if(args.length < 3){
			System.out.println("Usage: java -cp .. ru.oogis.dro.CreatedEmitter <path-to-project> <package> <path-to-drl-file>");
		}
		Collection errors = new ArrayList();
		System.out.println("load errors "+errors.size());
		if(errors.size() > 0){
			for (Iterator iter = errors.iterator(); iter.hasNext();) {
				Object element = iter.next();
				System.out.println("error: "+element);
			}
			return;
		}
		Collection pins = KB.c_Package.getInstances();
		for (Iterator iter = pins.iterator(); iter.hasNext();) {
			Instance pk = (Instance) iter.next();
			Object pnam = pk.getOwnSlotValue(KB.s_title);
			if(pnam.equals( args[1] )){
				CreatedEmitter ce = new CreatedEmitter(pk);
				ce.createAndEmitRules( args[2] );
				System.out.println(args[1]+" emmited to "+args[2]);
				return;
			}
		}
		System.out.println("Instance not found "+args[1]);
	}
}
