/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * Copyright (C) 2007 Ruslan P. Sorokin (sorokin@oogis.ru)
 * and Object-Oriented Geo-Information System Research Laboratory (http://www.oogis.ru)
 * All Rights Reserved.
 */
package ru.oogis.dro;

import edu.stanford.smi.protege.Application;
import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.InputStreamReader;
import java.lang.reflect.Constructor;
import java.net.URL;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.border.BevelBorder;
import javax.swing.filechooser.FileFilter;

import org.drools.FactHandle;
import org.drools.RuleBase;
import org.drools.RuleBaseFactory;
import org.drools.WorkingMemory;
import org.drools.compiler.DroolsError;
import org.drools.compiler.FunctionError;
import org.drools.compiler.PackageBuilder;
import org.drools.compiler.PackageBuilderErrors;
import org.drools.lang.descr.FunctionDescr;
import org.drools.rule.Package;

import ru.oogis.util.OntoJava;

import edu.stanford.smi.protege.model.Cls;
import edu.stanford.smi.protege.model.Instance;
import edu.stanford.smi.protege.model.Project;
import edu.stanford.smi.protege.model.SystemFrames;
import edu.stanford.smi.protege.model.WidgetDescriptor;
import edu.stanford.smi.protege.model.Slot;
import edu.stanford.smi.protege.ui.DisplayUtilities;
import edu.stanford.smi.protege.ui.ProjectManager;
import edu.stanford.smi.protege.ui.ProjectView;
import edu.stanford.smi.protege.util.CollectionUtilities;
import edu.stanford.smi.protege.util.ComponentFactory;
import edu.stanford.smi.protege.util.ModalDialog;
import edu.stanford.smi.protege.widget.AbstractListWidget;
import edu.stanford.smi.protege.widget.AbstractTabWidget;
import edu.stanford.smi.protege.widget.ClsWidget;
import edu.stanford.smi.protege.widget.SlotWidget;
import edu.stanford.smi.protege.widget.TabWidget;
import groovy.lang.Binding;
import groovy.lang.GroovyShell;
import java.io.PrintWriter;
import java.lang.reflect.Field;
import java.util.Date;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.swing.JTabbedPane;
import javax.swing.JTable;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import ru.oogis.util.OntoCompiler;

@SuppressWarnings("serial")
public class DroolsTab extends AbstractTabWidget {

    public static final String START_TIMER = "Start Timer";
    public static final String STOP_TIMER = "Stop Timer";
    private static Dimension windowSize = new Dimension(800, 600);
    private static Project project = null;
    private static Instance ruleBaseInstance = null;
    private static RuleBase ruleBase = null;
    private static WorkingMemory workingMemory = null;
    private static DroolsTab droTab = null;
    private ClsWidget ruleBaseWidget = null;
    private static boolean sortClasses = false;
    private static String ontopackage = null;
    @SuppressWarnings("unchecked")
    private static Map parameters = null;
    private static JLabel status = new JLabel();
    private static Component ruleBaseComponent = null;

    public void setup(WidgetDescriptor descr, Project prj) {
        super.setup(descr, prj);
        project = prj;
        KB.setKb(prj.getKnowledgeBase());
        droTab = this;
        System.gc();
    }

    public void initialize() {
        setLabel("Drools");
        JPanel rpanel = new JPanel();
        rpanel.setBorder(new BevelBorder(BevelBorder.LOWERED));
        setAnnotatedOrFirstRuleBaseInstance();
        addButtons(rpanel);
        ruleBaseComponent = createRuleBaseComponent();
        add(ruleBaseComponent, BorderLayout.CENTER);
        add(rpanel, BorderLayout.EAST);
        add(status, BorderLayout.SOUTH);
    }

    private static void setStatus(String mess){
        status.setText(mess);
    }

    @SuppressWarnings("unchecked")
    void setAnnotatedOrFirstRuleBaseInstance() {
        Cls rbcls = KB.c_RuleBase;
        if (rbcls != null) {
            Collection rbinss = rbcls.getInstances();
            if (!rbinss.isEmpty()) {
                ruleBaseInstance = findAnnotated(rbinss, null);
                if (ruleBaseInstance == null) {
                    ruleBaseInstance = (Instance) rbinss.iterator().next();
                }
            }
        }
    }

    Component createRuleBaseComponent() {
        if (ruleBaseInstance != null) {
            ruleBaseWidget = project.createRuntimeClsWidget(ruleBaseInstance);
            JPanel header = new JPanel();
            header.add(new JLabel("<html><body><h2>Rule Base Instance"));
            JPanel rbpanel = new JPanel(new BorderLayout());
            rbpanel.add(header, BorderLayout.NORTH);
            rbpanel.add((Component) ruleBaseWidget, BorderLayout.CENTER);
            return new JScrollPane(rbpanel);
        } else {
            return null;
        }
    }

    void addConsole(JSplitPane panel, JTextArea jta) {
        panel.setBottomComponent(new JScrollPane(jta));
    }

    void addButtons(JPanel panel) {
        JButton compOnbut = new JButton("Compile Ontology");
        compOnbut.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                compileOntology();
            }
        });
        JButton creaRBbut = new JButton("Create Rule Base");
        creaRBbut.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                createRuleBase();
            }
        });
        JButton dispRBbut = new JButton("Display Rule Base");
        dispRBbut.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                try {
                    displayRuleBase();
                } catch (Exception ex) {
                    Logger.getLogger(DroolsTab.class.getName()).log(Level.SEVERE, null, ex);
                }
            }
        });
        JButton chenRBbut = new JButton("Change Rule Base");
        chenRBbut.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                changeRuleBase();
            }
        });
        JButton mapClabut = new JButton("Assert Classes");
        mapClabut.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                assertClasses();
            }
        });
        JButton mapInsbut = new JButton("Assert Instances");
        mapInsbut.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                assertInstances();
            }
        });
        JButton assStrbut = new JButton("Assert Strings");
        assStrbut.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                assertStrings();
            }
        });
        JButton fireRubut = new JButton("Fire All Rules");
        fireRubut.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                fireAllRules();
            }
        });
        JButton displayWM = new JButton("Display Working Memory");
        displayWM.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                displayWorkingMemory();
            }
        });
        JButton clearWoMe = new JButton("Clear Working Memory");
        clearWoMe.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                clearWorkingMemory();
            }
        });
        JButton evalScrip = new JButton("Evaluate Scripts");
        evalScrip.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                evaluateScripts();
            }
        });
        JButton loadPkbut = new JButton("Load Packages");
        loadPkbut.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                loadPackages();
            }
        });
        JButton savePkbut = new JButton("Save Packages");
        savePkbut.addActionListener(new ActionListener() {

            public void actionPerformed(ActionEvent e) {
                savePackages();
            }
        });
        panel.setLayout(new GridLayout(13, 1));
        panel.add(compOnbut);
        panel.add(creaRBbut);
        panel.add(dispRBbut);
        panel.add(chenRBbut);
        panel.add(evalScrip);
        panel.add(mapClabut);
        panel.add(mapInsbut);
        panel.add(assStrbut);
        panel.add(fireRubut);
        panel.add(displayWM);
        panel.add(clearWoMe);
        panel.add(loadPkbut);
        panel.add(savePkbut);
    }

    @SuppressWarnings("unchecked")
    void compileOntology() {
        if (ruleBaseInstance != null) {
            ontopackage = (String) ruleBaseInstance.getOwnSlotValue(KB.s_onto_package);
            String classp = (String) ruleBaseInstance.getOwnSlotValue(KB.s_onto_classpath);
            String compil = (String) ruleBaseInstance.getOwnSlotValue(KB.s_onto_compiler);
            if (ontopackage != null && classp != null) {
                Collection topclasses = ruleBaseInstance.getOwnSlotValues(KB.s_onto_topclasses);
                Collection<String> mess = new ArrayList<String>();
                mess.add("Compilation of: "+new Date());
                for (Iterator iter = topclasses.iterator(); iter.hasNext();) {
                    Cls topclass = (Cls) iter.next();
                    mess.add("\nTop class: "+topclass.getName());
                    mess.add("----------------------------------------");
                    OntoJava oj = new OntoJava(classp, ontopackage, KB.kb);
                    mess.addAll(oj.createClasses(topclass.getName()));
                }
                displayMessages("Ontology Compilation Results", mess);
                logMessages("compilation.log", mess);
                setStatus("Compiled near "+(mess.size()-(topclasses.size()*2))+" classes into "+classp+"/"+ontopackage);
            }
        }
    }

    @SuppressWarnings("unchecked")
    void createRuleBase() {
        if (ruleBaseInstance != null) {
            ontopackage = (String) ruleBaseInstance.getOwnSlotValue(KB.s_onto_package);
            if (ontopackage != null) {
                Collection packages = ruleBaseInstance.getOwnSlotValues(KB.s_packages);
                if (!packages.isEmpty()) {
                    try {
                        if(createRuleBase(packages))
                            setStatus("Rule Base created.");
                        else
                            setStatus("Creating Rule Base errors!");
                    } catch (Exception e) {
                        e.printStackTrace();
                        setStatus("Creating Rule Base exception!");
                    }
                } else
                    setStatus("Packages not defined!");
            } else
                setStatus("Ontopackage not defined!");
        }
    }

    void displayRuleBase() throws Exception {
        if (ruleBase != null) {
            JTabbedPane jtb = new JTabbedPane();
            Package[] pp = ruleBase.getPackages();
            for (int i = 0; i < pp.length; i++) {
                Package p = pp[i];
                jtb.add(p.getName(), new PackagePanel(p));
            }
            jtb.setPreferredSize(new Dimension(800, 600));
            JFrame f = new JFrame("Rule Base");
            f.getContentPane().add(jtb);
            f.pack();
            f.setLocationRelativeTo(Application.getMainWindow());
            f.setVisible(true);
        }
    }

    void changeRuleBase() {
        Instance selected = DisplayUtilities.pickInstance(
                this,
                CollectionUtilities.createCollection(KB.c_RuleBase),
                "Select Rule Base Instance");
        if (selected != null) {
            ruleBaseInstance = selected;
            remove(ruleBaseComponent);
            ruleBaseComponent = createRuleBaseComponent();
            add(ruleBaseComponent, BorderLayout.CENTER);
        }
    }

    @SuppressWarnings("unchecked")
    void evaluateScripts() {
        if (ruleBaseInstance != null) {
            Collection scripts = ruleBaseInstance.getOwnSlotValues(KB.s_scripts);
            SlotWidget slotWidget = ruleBaseWidget.getSlotWidget(KB.s_scripts);
            if (slotWidget instanceof AbstractListWidget) {
                scripts = ((AbstractListWidget) slotWidget).getSelection();
            }
            String scriptTitle = "";
            try {
                int ns = 0;
                for (Iterator iter = scripts.iterator(); iter.hasNext();) {
                    Instance scriptInst = (Instance) iter.next();
                    scriptTitle = (String) scriptInst.getOwnSlotValue(KB.s_title);
                    evaluateScript(scriptInst);
                    ns++;
                }
                setStatus(ns + " scripts evaluated.");
            } catch (Exception e) {
                e.printStackTrace();
                JOptionPane.showMessageDialog(this,
                        "Exception while evaluating script " + scriptTitle,
                        "Script Evaluation", JOptionPane.ERROR_MESSAGE);
            }
        }
    }

    @SuppressWarnings("unchecked")
    void assertClasses() {
        if (ruleBaseInstance != null && workingMemory != null) {
            Collection classes = ruleBaseInstance.getOwnSlotValues(KB.s_assert_classes);
            SlotWidget slotWidget = ruleBaseWidget.getSlotWidget(KB.s_assert_classes);
            if (slotWidget instanceof AbstractListWidget) {
                classes = ((AbstractListWidget) slotWidget).getSelection();
            }
            for (Iterator iter = classes.iterator(); iter.hasNext();) {
                Cls cls = (Cls) iter.next();
                try {
                    assertClass(cls);
                    setStatus("Asserted class : " + cls + " instances");
                } catch (Exception e) {
                    e.printStackTrace();
                    JOptionPane.showMessageDialog(this,
                            "Exception while asserting class " + cls
                            + " instances!", "Class Assertion",
                            JOptionPane.ERROR_MESSAGE);
                }
            }
        }
    }

    @SuppressWarnings("unchecked")
    void assertInstances() {
        if (ruleBaseInstance != null && workingMemory != null) {
            Collection instances = ruleBaseInstance.getOwnSlotValues(KB.s_assert_instances);
            SlotWidget slotWidget = ruleBaseWidget.getSlotWidget(KB.s_assert_instances);
            if (slotWidget instanceof AbstractListWidget) {
                instances = ((AbstractListWidget) slotWidget).getSelection();
            }
            for (Iterator iter = instances.iterator(); iter.hasNext();) {
                Instance instance = (Instance) iter.next();
                try {
                    workingMemory.insert(mapInstance(instance));
                    setStatus("Asserted: " + instance);
                } catch (Exception e) {
                    e.printStackTrace();
                    JOptionPane.showMessageDialog(this,
                            "Exception while asserting instance " + instance,
                            "Instance Assertion", JOptionPane.ERROR_MESSAGE);
                    return;
                }
            }
        }
    }

    @SuppressWarnings("unchecked")
    void assertStrings() {
        if (ruleBaseInstance != null && workingMemory != null) {
            Collection strings = ruleBaseInstance.getOwnSlotValues(KB.s_assert_strings);
            SlotWidget slotWidget = ruleBaseWidget.getSlotWidget(KB.s_assert_strings);
            if (slotWidget instanceof AbstractListWidget) {
                strings = ((AbstractListWidget) slotWidget).getSelection();
            }
            for (Iterator iter = strings.iterator(); iter.hasNext();) {
                Object stringObject = iter.next();
                workingMemory.insert(stringObject);
                setStatus("Asserted: " + stringObject);
            }
        }
    }

    void fireAllRules() {
        if (workingMemory != null) {
            workingMemory.fireAllRules();
        }
        setStatus("All rules fired.");
    }

    @SuppressWarnings("unchecked")
    void displayWorkingMemory() {
        if (workingMemory != null && ruleBaseInstance!=null){
            Instance wmi = (Instance) ruleBaseInstance.getOwnSlotValue(KB.s_working_memory);
            if(wmi != null) {
                WorkingMemoryInspector wmInspector = new WorkingMemoryInspector(workingMemory, wmi);
                wmInspector.setPreferredSize(new Dimension(800,600));
                JFrame f = new JFrame("Working Memory Snapshot");
                f.getContentPane().add(wmInspector);
                f.pack();
                f.setLocationRelativeTo(Application.getMainWindow());
                f.setVisible(true);
            } else
                setStatus("Working Memory Instance not found!");
        }
    }

    @SuppressWarnings("unchecked")
    void clearWorkingMemory() {
        if (workingMemory != null) {
            int i = JOptionPane.showConfirmDialog(this, "Clear Working Memory?");
            if (i != JOptionPane.YES_OPTION) {
                return;
            }
            int cnt = 0;
            for (Iterator iter = workingMemory.iterateFactHandles(); iter.hasNext();) {
                FactHandle fhandle = (FactHandle) iter.next();
                workingMemory.retract(fhandle);
                cnt++;
            }
            setStatus("Retracted: " + cnt);
        }
    }

    @SuppressWarnings("unchecked")
    void loadPackages() {
        boolean errors = false;
        Collection selpacks = DisplayUtilities.pickInstances(this, KB.kb,
                CollectionUtilities.createCollection(KB.c_Package), "Select Packages to Load");
        for (Iterator iter = selpacks.iterator(); iter.hasNext();) {
            Instance pkginst = (Instance) iter.next();
            Object path = pkginst.getOwnSlotValue(KB.s_path);
            if (path != null) {
                DrlParser dp = new DrlParser(KB.kb, pkginst);
                try {
                    dp.parse((String) path);
                } catch (Exception e) {
                    e.printStackTrace();
                    errors = true;
                }
                int errs = dp.getErrors().size();
                if (errs > 0) {
                    setStatus("Loading package from " + path + " eroors "+errs);
                    errors = true;
                    for (Iterator iter2 = dp.getErrors().iterator(); iter.hasNext();) {
                        Object element = iter2.next();
                        System.out.println("Loading package from " + path + " error: " + element);
                    }
                } else {
                    setStatus("Packages loaded.");
                }
            }
        }
    }

    @SuppressWarnings("unchecked")
    void savePackages() {
        boolean errors = false;
        Collection selpacks = DisplayUtilities.pickInstances(this, KB.kb,
                CollectionUtilities.createCollection(KB.c_Package), "Select Packages to Save");
        for (Iterator iter = selpacks.iterator(); iter.hasNext();) {
            Instance pkginst = (Instance) iter.next();
            Object titl = pkginst.getOwnSlotValue(KB.s_title);
            Object path = pkginst.getOwnSlotValue(KB.s_path);
            if (path != null) {
                CreatedEmitter ce = new CreatedEmitter(pkginst);
                try {
                    ce.createAndEmitRules((String) path);
                } catch (Exception e) {
                    e.printStackTrace();
                    errors = true;
                }
            }
        }
        if (errors) {
            setStatus("Save packages errors");
        } else {
            setStatus("Packages saved");
        }
    }

    @SuppressWarnings("unchecked")
    public static void startScenario(String title, Map params) throws Exception {
        if (ruleBaseInstance != null && workingMemory != null) {
            Instance sceninst = DroolsTab.findForTitle("Scenario", title);
            if (sceninst != null) {
                ProtegeInstance instance = DroolsTab.mapInstance(sceninst);
                instance.setCtxmap(collectionsToInstances(params));
                workingMemory.insert(instance);
                workingMemory.fireAllRules();
            }
        }
    }

    @SuppressWarnings("unchecked")
    private static Map collectionsToInstances(Map params) {
        for (Object key : params.keySet()) {
            Object val = params.get(key);
            if (val instanceof Collection) {
                String name = (String) ((Collection<String>) val).iterator().next();
                Instance obj = DroolsTab.findForSlotValue("MapOb", "label", name);
                if (obj == null) {
                    obj = KB.kb.createInstance(null, KB.kb.getCls("MapOb"));
                    obj.setOwnSlotValue(KB.kb.getSlot("label"), name);
                }
                params.put(key, obj);
            }
        }
        return params;
    }

    @SuppressWarnings("unchecked")
    public static Map scenarioContext(String title) {
        if (ruleBaseInstance != null && workingMemory != null) {
            Instance sceninst = DroolsTab.findForTitle("Scenario", title);
            if (sceninst != null) {
                Instance ctx = (Instance) sceninst.getOwnSlotValue(KB.s_context);
                if (ctx != null) {
                    Map map = new HashMap();
                    Collection<Instance> pairs = ctx.getOwnSlotValues(KB.s_pairs);
                    for (Instance pair : pairs) {
                        Instance var = (Instance) pair.getOwnSlotValue(KB.s_variable);
                        String vartit = (String) var.getOwnSlotValue(KB.s_title);
                        if (pair.getDirectType().equals(KB.c_ValuePair)) {
                            String value = (String) pair.getOwnSlotValue(KB.s_value);
                            map.put(vartit, value);
                        } else if (pair.getDirectType().equals(KB.c_ObjectPair)) {
                            Instance obj = (Instance) pair.getOwnSlotValue(KB.s_object);
                            String label = (String) obj.getOwnSlotValue(KB.kb.getSlot("label"));
                            Collection labcol = new ArrayList();
                            labcol.add(label);
                            map.put(vartit, labcol);
                        }
                    }
                    return map;
                }
            }
        }
        return null;
    }

    public static Object evaluateScript(String scriptInstTitle) throws Exception {
        Instance scriptInst = findForTitle(KB.c_Script.getName(), scriptInstTitle);
        return evaluateScript(scriptInst);
    }

    public static Object evaluateScript(Instance scriptInst) throws Exception {
        String script = (String) scriptInst.getOwnSlotValue(KB.s_source);
        return evaluateGroovyScript(script);
    }

    public static Object evaluateGroovyScript(String script) throws Exception {
        Binding binding = new Binding();
        binding.setVariable("kb", KB.kb);
        binding.setVariable("prj", project);
        binding.setVariable("dro", droTab);
        binding.setVariable("rb", ruleBase);
        binding.setVariable("wm", workingMemory);
        GroovyShell shell = new GroovyShell(
                droTab.getClass().getClassLoader(), binding);
        return shell.evaluate(script);
    }

    public static Package createPackage(Instance pinst) throws Exception {
        Object title = pinst.getOwnSlotValue(KB.s_title);
        PackageBuilder pb = new PackageBuilder();
        CreatedEmitter ce = new CreatedEmitter(pinst);
        ce.addPackageToBuilder(pb);
        if(checkErrors(pb, title)){
            return pb.getPackage();
        } else {
            return null;
        }
    }

    private static boolean checkErrors(PackageBuilder pb, Object title) {
        PackageBuilderErrors errors = pb.getErrors();
        if (!errors.isEmpty()) {
            DroolsError[] derr = errors.getErrors();
            for (int i = 0; i < derr.length; i++) {
                System.out.println("In package " + title + " error: " + derr[i].toString());
                int[] elines = derr[i].getLines();
                for(int j=0; j<elines.length; j++)
                    System.out.println("        Line: "+elines[j]);
                if (derr[i] instanceof FunctionError) {
                    FunctionDescr fdesc = ((FunctionError) derr[i]).getFunctionDescr();
                    System.out.println(" function name: " + fdesc.getName());
                }
            }
            setStatus("Package errrors!");
            return false;
        }
        return true;
    }

    @SuppressWarnings("unchecked")
    public static boolean createRuleBase(Collection packages) throws Exception {
        ruleBase = RuleBaseFactory.newRuleBase();
        boolean ok = true;
        for (Iterator iter = packages.iterator(); iter.hasNext();) {
            Instance pinst = (Instance) iter.next();
            Package pkg = createPackage(pinst);
            if (pkg != null) {
                ruleBase.addPackage(pkg);
            } else {
                ok = false;
            }
        }
        if(!ok)
            return false;
        workingMemory = ruleBase.newStatefulSession();
        for (Iterator iter = packages.iterator(); iter.hasNext();) {
            Instance pinst = (Instance) iter.next();
            Object ptitle = pinst.getOwnSlotValue(KB.s_title);
            Collection globals = pinst.getOwnSlotValues(KB.s_globals);
            for (Iterator iterator = globals.iterator(); iterator.hasNext();) {
                Instance global = (Instance) iterator.next();
                Object title = global.getOwnSlotValue(KB.s_title);
                Object clazz = global.getOwnSlotValue(KB.s_class);
                Object value = global.getOwnSlotValue(KB.s_value);
                Object val = null;
                Class clas = Class.forName((String) clazz);
                if (clas != null) {
                    Constructor constructor;
                    try {
                        constructor = clas.getConstructor(new Class[]{String.class});
                        val = constructor.newInstance(new Object[]{value});
                    } catch (NoSuchMethodException nsme) {
                        val = evaluateGroovyScript((String) value);
                    }
                    if (val != null) {
                        workingMemory.setGlobal((String) title, val);
                    } else {
                        System.out.println("In Package " + ptitle + " global "
                                + title + " value " + value + " not created!");
                        setStatus("Global error!");
                        ok = false;
                    }
                } else {
                    System.out.println("\nIn Package " + ptitle + " global " + title
                            + " class " + clazz + " not found!");
                    setStatus("Global error!");
                    ok = false;
                }
            }
        }
        return ok;
    }

    @SuppressWarnings("unchecked")
    public static Instance findAnnotated(Collection inss, String text) {
        if (inss == null && text == null) {
            return null;
        }
        SystemFrames sf = KB.kb.getSystemFrames();
        Cls acls = sf.getAnnotationCls();
        Slot tslot = sf.getAnnotationTextSlot();
        Slot islot = sf.getAnnotatedInstanceSlot();
        Collection ainss = acls.getInstances();
        for (Iterator iter = ainss.iterator(); iter.hasNext();) {
            Instance ainst = (Instance) iter.next();
            Object txt = ainst.getOwnSlotValue(tslot);
            Object inst = ainst.getOwnSlotValue(islot);
            if ((inss == null || inss.contains(inst))
                    && (text == null || text.startsWith((String) txt))) {
                return (Instance) inst;
            }
        }
        return null;
    }

    @SuppressWarnings("unchecked")
    public static void assertClass(Cls cls) throws Exception {
        Collection instances = cls.getInstances();
        for (Iterator iter = instances.iterator(); iter.hasNext();) {
            Instance instance = (Instance) iter.next();
            workingMemory.insert(mapInstance(instance));
        }
    }

    public static ProtegeInstance mapInstance(Instance instance) throws Exception {
        ProtegeInstance pinst = (ProtegeInstance) OntoJava.map(instance, ontopackage, KB.kb);
        if (pinst == null) {
            throw new Exception("OntoJava Exception");
        } else {
            return pinst;
        }
    }

    public static Instance findForTitle(String clsname, String title) {
        return findForSlotValue(clsname, "title", title);
    }

    @SuppressWarnings("unchecked")
    public static Instance findForSlotValue(String clsname, String slotname, Object slotvalue) {
        Collection inss;
        if (clsname != null) {
            Cls cls = KB.kb.getCls(clsname);
            inss = cls.getInstances();
        } else {
            inss = KB.kb.getInstances();
        }
        Slot slot = KB.kb.getSlot(slotname);
        if (slot != null) {
            for (Iterator iter = inss.iterator(); iter.hasNext();) {
                Instance inst = (Instance) iter.next();
                Object val = inst.getOwnSlotValue(slot);
                if (val != null && val.equals(slotvalue)) {
                    return inst;
                }
            }
        }
        return null;
    }

    public static TabWidget getTab(String className) {
        ProjectManager pm = ProjectManager.getProjectManager();
        ProjectView pv = pm.getCurrentProjectView();
        return pv.getTabByClassName(className);
    }

    public static void printout(String message) {
        System.out.println(message);
    }

    public static boolean showModal(String title, Component comp, String comment) {
        final JPanel panel = new JPanel(new BorderLayout());
        JTextArea jta = new JTextArea(comment);
        jta.setBackground(new Color(1f, 1f, 0.8f));
        Font font = jta.getFont();
        jta.setFont(new Font(font.getFontName(), Font.ITALIC, font.getSize()));
        jta.setLineWrap(true);
        jta.setWrapStyleWord(true);
        panel.add(jta, BorderLayout.NORTH);
        panel.add(comp, BorderLayout.CENTER);
        panel.validate();
        Dimension size = panel.getPreferredSize();
        if (size.width <= windowSize.width && size.height <= windowSize.height) {
            comp = panel;
        } else {
            JScrollPane jsp = new JScrollPane(panel);
            jsp.setPreferredSize(windowSize);
            comp = jsp;
        }

        ModalDialog.CloseCallback callback = new ModalDialog.CloseCallback() {

            public boolean canClose(int result) {
                return true;
            }
        };

        int ret = ModalDialog.showDialog(null, comp, title,
                ModalDialog.MODE_OK_CANCEL, callback);
        return (ret == ModalDialog.OPTION_OK);
    }

    public static Dimension getWindowSize() {
        return windowSize;
    }

    public static void setWindowSize(int width, int height) {
        windowSize = new Dimension(width, height);
    }

    public static DroolsTab getDroTab() {
        return droTab;
    }

    public static RuleBase getRuleBase() {
        return ruleBase;
    }

    public static Instance getRuleBaseInstance() {
        return ruleBaseInstance;
    }

    public static WorkingMemory getWorkingMemory() {
        return workingMemory;
    }

    @SuppressWarnings("unchecked")
    public static void main(String[] args) throws Exception {
        String path = null;
        if (args.length < 1) {
            JFileChooser chooser = new JFileChooser(".");
            chooser.setDialogTitle("Open Protege Project");
            chooser.setFileFilter(new FileFilter() {

                public boolean accept(File f) {
                    if (f != null) {
                        if (f.isDirectory()) {
                            return true;
                        }
                        String fname = f.getName();
                        int pind = fname.lastIndexOf('.');
                        if (pind < 0) {
                            return false;
                        }
                        return fname.substring(pind).toLowerCase().equals(
                                ".pprj");
                    }
                    return false;
                }

                @Override
                public String getDescription() {
                    return "*.pprj";
                }
            });
            int returnVal = chooser.showOpenDialog(null);
            if (returnVal == JFileChooser.APPROVE_OPTION) {
                path = chooser.getSelectedFile().getPath();
            }
        } else {
            System.out.println("INITIAL_PROJECT "+args[0]);
            File file = new File(args[0]);
            if (file.exists()) {
                path = file.getPath();
            }
        }
        System.out.println("Project path: "+path);
        if (path == null) {
            return;
        }
        Collection errors = new ArrayList();
        Project prj = Project.loadProjectFromFile(path, errors);
        System.out.println("  load errors " + errors.size());
        if (errors.size() > 0) {
            for (Iterator iter = errors.iterator(); iter.hasNext();) {
                Object element = iter.next();
                System.out.println("  error: " + element);
            }
            return;
        }
        final ProjectManager projectManager = ProjectManager.getProjectManager();
        JFrame mainFrame = ComponentFactory.createMainFrame();
        mainFrame.addWindowListener(new WindowAdapter() {

            public void windowClosing(WindowEvent event) {
                projectManager.exitApplicationRequest();
            }
        });
        projectManager.setRootPane(mainFrame.getRootPane());
        projectManager.setCurrentProject(prj, false);
        mainFrame.pack();
        mainFrame.setVisible(true);
    }

    public static String getOntopackage() {
        return ontopackage;
    }

    public static void setOntopackage(String ontopackage) {
        DroolsTab.ontopackage = ontopackage;
    }

    @SuppressWarnings("unchecked")
    public static Map getParameters() {
        return parameters;
    }

    @SuppressWarnings("unchecked")
    public static void setParameters(Map parameters) {
        DroolsTab.parameters = parameters;
    }
    private static DefaultListModel advices;
    private static DefaultListModel pages;
    private static JList advicesJList;
    private static JFrame advicesJFrame;

    public static void showESMessage(Object message, Map<String, String> pageMap, int category) {
        if (advicesJFrame == null) {
            advicesJFrame = new JFrame("Expert System Messages");
            advicesJFrame.addWindowListener(new java.awt.event.WindowAdapter() {

                public void windowClosing(java.awt.event.WindowEvent evt) {
                    advicesJFrame.dispose();
                    advicesJFrame = null;
                }
            });
            pages = new DefaultListModel();
            advices = new DefaultListModel();
            advicesJList = new javax.swing.JList(advices);
            advicesJList.setFont(new java.awt.Font("Monospaced",
                    java.awt.Font.PLAIN, 12));
            advicesJList.setBackground(java.awt.Color.BLACK);
            advicesJList.setForeground(java.awt.Color.GREEN);
            advicesJList.addMouseListener(
                    new MouseAdapter() {

                        @SuppressWarnings("unchecked")
                        public void mouseClicked(MouseEvent e) {
                            int index = advicesJList.locationToIndex(e.getPoint());
                            Map<String, String> map = (Map<String, String>) pages.get(index);
                            String urls = (String) map.get("url");
                            if (urls != null) {
                                String text = "";
                                try {
                                    URL url = new java.net.URL(urls);
                                    BufferedReader br = new BufferedReader(
                                            new InputStreamReader(
                                            new FileInputStream(url.getPath()), "UTF-8"));
                                    while (br.ready()) {
                                        text += br.readLine();
                                    }
                                    br.close();
                                } catch (Exception e1) {
                                    // TODO Auto-generated catch block
                                    e1.printStackTrace();
                                }
                                Set keys = map.keySet();
                                for (Object key : keys) {
                                    String value = map.get(key);
                                    if (((String) key).startsWith("?") && value != null) {
                                        text = text.replace((String) key, value);
                                    }
                                }
                                JLabel lab = new JLabel(text);
                                JScrollPane jsp = new JScrollPane(lab);
                                JFrame jf = new JFrame("Detailed Message");
                                jf.getContentPane().add(jsp);
                                jf.pack();
                                jf.setVisible(true);
                            }

                        }
                    });
            javax.swing.JScrollPane ajsp = new javax.swing.JScrollPane(
                    advicesJList);
            ajsp.setPreferredSize(new java.awt.Dimension(800, 120));
            JButton clbut = new JButton("Clear");
            clbut.addActionListener(new ActionListener() {

                public void actionPerformed(ActionEvent e) {
                    ((DefaultListModel) advicesJList.getModel()).clear();
                }
            });
            advicesJFrame.getContentPane().add(ajsp);
            advicesJFrame.pack();
        }
        switch (category) {
            case 1:
                message = "<html><body color='#4444FF'>" + message
                        + "</html></body>";
                break;
            case 2:
                message = "<html><body color='#FFFF00'>" + message
                        + "</html></body>";
                break;
            case 3:
                message = "<html><body color='#FF4444'>" + message
                        + "</html></body>";
                break;
        }
        advices.add(0, message);
        if (pageMap != null) {
            pages.add(0, pageMap);
        } else {
            pages.add(0, new HashMap<String, String>());
        }
        advicesJFrame.setVisible(true);
        advicesJFrame.toFront();
    }
    public static void displayObject(Object obj, Object header) throws Exception{
        if(obj!=null && obj!=displ_obj){
            if(obj instanceof Number)
                ;
            else if(obj instanceof Instance)
                ProjectManager.getProjectManager().getCurrentProject().show((Instance)obj);
            else {
                Component comp;
                String head;
                if(obj instanceof String){
                    head = "String";
                    comp = new JTextArea((String)obj);
                    comp.setPreferredSize(new Dimension(400, 200));
                } else {
                    final JTable tab;
                    final Object[][] data;
                    String col0;
                    if(obj instanceof Collection){
                        Collection col = (Collection) obj;
                        data = new Object[col.size()][2];
                        int i = 0;
                        for (Object ob : col) {
                            data[i] = new Object[]{i, ob};
                            i++;
                        }
                        col0 = "index";
                        head = "Collection";
                    } else if(obj instanceof Map){
                        Map map = (Map) obj;
                        data = new Object[map.size()][2];
                        int i = 0;
                        for (Object key : map.keySet()) {
                            data[i] = new Object[]{key, map.get(key)};
                            i++;
                        }
                        col0 = "key";
                        head = "Map";
                    } else {
                        Class cls = obj.getClass();
                        Field[] fields = cls.getFields();
                        data = new Object[fields.length][2];
                        for (int i = 0; i < fields.length; i++) {
                            data[i] = new Object[]{fields[i].getName(), fields[i].get(obj)};
                        }
                        col0 = "field";
                        head = "Object: "+cls.getName();
                    }
                    tab = new JTable(data, new String[]{col0,"value"});
                    tab.getSelectionModel().addListSelectionListener(new ListSelectionListener() {
                        @Override
                        public void valueChanged(ListSelectionEvent event){
                            try {
                                int row = tab.getSelectedRow();
                                if(row>=0){
                                    displayObject(data[row][1], data[row][0]);
                                }
                            } catch (Exception ex) {
                                Logger.getLogger(WorkingMemoryInspector.class.getName()).log(Level.SEVERE, null, ex);
                            }
                        }
                    });
                    comp = tab;
                    comp.setPreferredSize(new Dimension(400, 700));
                }
                JFrame f = new JFrame((String) (header!=null ? header : head));
                f.getContentPane().add(new JScrollPane(comp));
                f.pack();
                f.setLocationRelativeTo(Application.getMainWindow());
                f.setVisible(true);
                displ_obj = obj;
            }
        }
    }
    private static Object displ_obj;

    public static void displayMessages(String title, Collection<String> messages){
        JTextArea jta = new JTextArea();
        for (String string : messages) {
            jta.append(string+"\n");
        }
        JFrame f = new JFrame(title);
        JScrollPane jsp = new JScrollPane(jta);
        jsp.setPreferredSize(new Dimension(600, 800));
        f.getContentPane().add(jsp);
        f.pack();
        f.setVisible(true);
        f.setLocationRelativeTo(droTab);
    }

    public static void logMessages(String file, Collection<String> messages) {
        PrintWriter log = null;
        try {
            log = new PrintWriter(file);
            for (String string : messages) {
                log.println(string);
            }
            log.close();
        } catch (FileNotFoundException ex) {
            Logger.getLogger(DroolsTab.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
            log.close();
        }
    }
}
