/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * Copyright (C) 2007 Ruslan P. Sorokin (sorokin@oogis.ru)
 * and Object-Oriented Geo-Information System Research Laboratory (http://www.oogis.ru)
 * All Rights Reserved.
 */
package ru.oogis.util;

import java.lang.reflect.Field;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import ru.oogis.dro.ProtegeInstance;

import edu.stanford.smi.protege.model.Instance;
import edu.stanford.smi.protege.model.KnowledgeBase;
import edu.stanford.smi.protege.model.Cls;
import edu.stanford.smi.protege.model.Project;
import edu.stanford.smi.protege.model.Slot;
import edu.stanford.smi.protege.model.ValueType;

public class OntoJava {
	
	public static final String TAB = "    ";
	public static final String COLLECTION = "java.util.Collection";
	public static final String SUPERCLASS = ProtegeInstance.class.getName();
	
	private String destin_path;
	private String pakage;
	private KnowledgeBase kb;
        private OntoCompiler compiler;
	
	public OntoJava(String destin_path, String pakage, KnowledgeBase kb) {
		this.destin_path = destin_path;
		this.pakage = pakage;
		this.kb = kb;
                compiler = new GroovyCompiler();

	}

	@SuppressWarnings("unchecked")
	public String classCode(String pakage, Cls cls, Cls scl){
		String ccode = headerCode(pakage, cls, scl);
		Collection<Slot> ts = cls.getDirectTemplateSlots();
		if(scl == null)
			ts = cls.getTemplateSlots();
		for (Iterator iter = ts.iterator(); iter.hasNext();) {
			Slot slot = (Slot) iter.next();
			ccode += fieldCode(cls, slot);
		}
		for (Iterator iter = ts.iterator(); iter.hasNext();) {
			Slot slot = (Slot) iter.next();
			ccode += getterCode(cls, slot);
		}
		for (Iterator iter = ts.iterator(); iter.hasNext();) {
			Slot slot = (Slot) iter.next();
			ccode += setterCode(cls, slot);
		}
		ccode += toStringCode(cls, scl);
		ccode += toExtendedStringCode(cls, scl);
		return ccode + footerCode();
	}
	
	public String headerCode(String pakage, Cls cls, Cls scl){
		String hcode = "package "+pakage+";\n"+"public class "+cls.getName();
		if(scl != null)
			hcode += " extends "+scl.getName();
		else 
			hcode += " extends "+SUPERCLASS;
		return hcode+" {\n";
	}
	
	public String footerCode(){
		return "}\n";
	}
	
	public String fieldCode(Cls cls, Slot slot){
		ValueType vt = cls.getTemplateSlotValueType(slot);
		String typ = vt.getJavaType().getName();
		if(slot.getAllowsMultipleValues())
			typ = COLLECTION;
		String fld = slot.getName().replace('-', '_');
		return TAB+"public "+typ+" "+fld+";\n";
	}
	
	public String getterCode(Cls cls, Slot slot){
		ValueType vt = cls.getTemplateSlotValueType(slot);
		String typ = vt.getJavaType().getName();
		if(slot.getAllowsMultipleValues())
			typ = COLLECTION;
		String fld = slot.getName().replace('-', '_');
		String met = "get"+fld.substring(0, 1).toUpperCase()+fld.substring(1);
		return TAB+"public "+typ+" "+met+"(){\n"+TAB+TAB+"return "+fld+";\n"+TAB+"}\n";
	}

	public String setterCode(Cls cls, Slot slot){
		ValueType vt = cls.getTemplateSlotValueType(slot);
		String typ = vt.getJavaType().getName();
		if(slot.getAllowsMultipleValues())
			typ = COLLECTION;
		String fld = slot.getName().replace('-', '_');
		String met = "set"+fld.substring(0, 1).toUpperCase()+fld.substring(1);
		return TAB+"public void "+met+"("+typ+" "+fld+"){\n"+TAB+TAB+"this."+fld+" = "+fld+";\n"+TAB+"}\n";
	}
	
	@SuppressWarnings("unchecked")
	public String toStringCode(Cls cls, Cls scl){
		if(scl != null)
			return ""; // Only top class has toString method 
		else {
			String tscode = TAB+"public String toString(){\n"+TAB+TAB+"return super.toString()+\"";
			Collection<Slot> ts = cls.getTemplateSlots();
			for (Iterator iter = ts.iterator(); iter.hasNext();) {
				Slot slot = (Slot) iter.next();
				String fld = slot.getName().replace('-', '_');
				if(!fld.equals("context"))
					tscode += "\\t\"+"+fld+"+\"";
			}
			tscode += "\";\n"+TAB+"}\n";
			return tscode;
		} 
	}
	
	@SuppressWarnings("unchecked")
	public String toExtendedStringCode(Cls cls, Cls scl){
		String tscode = TAB+"public String toExtendedString(){\n"+TAB+TAB+"return super.toExtendedString()+\"";
		Collection<Slot> ts = cls.getDirectTemplateSlots();
		if(scl == null)
			ts = cls.getTemplateSlots();
		for (Iterator iter = ts.iterator(); iter.hasNext();) {
			Slot slot = (Slot) iter.next();
			String fld = slot.getName().replace('-', '_');
			tscode += TAB+fld+" \"+"+fld+"+\"\\n";
		}
		tscode += "\";\n"+TAB+"}\n";
		return tscode;
	}
	
	public Collection<String> createClasses(String top_class) {
            Cls cls = kb.getCls(top_class);
            if (cls != null) {
                Collection<String> mess = new ArrayList();
                mess.addAll(createClasses(cls, null));
                return mess;
            }
            return null;
	}
	
	@SuppressWarnings("unchecked")
	public Collection<String> createClasses(Cls cls, Cls scl) {
                ArrayList mess = new ArrayList();
		mess.add(process(cls, scl));
		Collection dsc = cls.getDirectSubclasses();
		for (Iterator iter = dsc.iterator(); iter.hasNext();) {
			Cls c = (Cls) iter.next();
			mess.addAll(createClasses(c, cls));
		}
		return mess;
	}
	
	public String process(Cls cls, Cls scl) {
		String source = classCode(pakage, cls, scl);
		String title = cls.getName();
                return compiler.compile(title, source, destin_path);
	}
	
	@SuppressWarnings("unchecked")
	public static Object map(Instance inst, String pakage, KnowledgeBase kb){
		Cls cls = inst.getDirectType();
		String className = pakage+"."+cls.getName();
		try {
			Class clazz = Class.forName(className);
			Object obj = clazz.newInstance();
			Field[] fields = clazz.getFields();
			for (int i = 0; i < fields.length; i++) {
				String fname = fields[i].getName();
				if(fname.equals("instance"))
					fields[i].set(obj, inst);
				else if(!fname.equals("ctxmap")) {
					Slot slot = kb.getSlot(fname);
					Collection<Slot> slots = cls.getTemplateSlots();
					if(!slots.contains(slot) && fname.contains("_"))
						slot = kb.getSlot(fname.replace('_', '-'));
					if(slots.contains(slot)){
						Object value = inst.getOwnSlotValue(slot);
						if(slot.getAllowsMultipleValues())
							value = inst.getOwnSlotValues(slot);
						fields[i].set(obj, value);
					}
				}
			}
			return obj;
		} catch (ClassNotFoundException e) {
			System.out.println("OntoJava.map: could not find class "+className);
			return null;
		} catch (InstantiationException e) {
			System.out.println("OntoJava.map: could not create instance of class "+className);
			return null;
		} catch (IllegalAccessException e) {
			System.out.println("OntoJava.map: IllegalAccessException for class "+className);
			e.printStackTrace();
			return null;
		}
	}

	@SuppressWarnings("unchecked")
	public static void main(String[] args) throws Exception {
		/*
		Project p = Project.loadProjectFromFile("/home/ru/JBRS-3.0/projects/Scenario.pprj", new ArrayList());
		KnowledgeBase kb = p.getKnowledgeBase();
		OntoJava oj = new OntoJava(
				"/home/ru/JBRS-3.0/classes", 
				"ru.oogis.pins", 
				kb);
		oj.createClasses("Pair");
		Cls pcls = kb.getCls("Search");
		Collection iss = pcls.getInstances();
		for (Iterator iter = iss.iterator(); iter.hasNext();) {
			Instance inst = (Instance) iter.next();
			Object obj = map(inst, "ru.oogis.pins", kb);
			System.out.println(((ProtegeInstance)obj).toExtendedString());
		}
		*/
		if(args.length < 6)
			System.out.println("Obligatory arguments: project_path destination_path package top_class");
		else {
			Project p = Project.loadProjectFromFile(args[0], new ArrayList());
			KnowledgeBase kb = p.getKnowledgeBase();
			OntoJava oj = new OntoJava(args[1], args[2], kb);
			oj.createClasses(args[3]);
		}
	}

}
