/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * Copyright (C) 2004 Sorokin R.P. (sorokinru@iias.spb.su)
 * and Object-oriented Geoinformatics Systems Laboratory (http://niggis.iias.spb.su)
 * All Rights Reserved.
 */

package ru.igis.omtab;

import java.awt.event.*;
import javax.swing.*;
/**
 * Time pulses generator
 * @author srp
 */
public class Clock implements ActionListener { 
    // cummulative values
    private static long second0 = 0;
    private static long minute0 = 0;
    private static long hour0 = 0;
    // current values
    private static long second = 0;
    private static long minute = 0;
    private static long hour = 0;
    private static long day = 0;
    // current time in ours
    private static double currentTime = 0; 
    // current time in milliseconds
    private long clock = 0;
    private float timeScale = 1;
    private RuNA runa;
    private RunaPanel runaPanel;
    private RunaControlPanel runaControlPanel;
    
    /**
     * Creates a new instance of Clock
     * @param runa - RuNA object
     */
    public Clock(RuNA runa) {
        this.runa = runa;
        Timer timer = runa.getTimer();
        timer.addActionListener(this);
        runaPanel = (RunaPanel)runa.getGUI();
    }

    /**
     * Timer events handler
     * @param ae - Event object
     */    
    public void actionPerformed(ActionEvent ae) {
        manageClock();
    }
    
    private void manageClock(){
        int interval = runa.getUpdateInterval();
        int scaledInterval = Math.round(interval * timeScale);
        if(interval>=1)
            clock += scaledInterval;
        else
            clock += interval;  // if Time Scale < 1:1000 go to 1:1
        currentTime = (double)clock/3600000d;
        long secs = clock/1000;
        long mins = secs/60;
        long hrs = mins/60;
        day = hrs/24; 
        second = secs % 60;
        minute = mins % 60;
        hour = hrs % 24;
        if (runaPanel != null){ 
            runaPanel.setClock("D-"+day+" "+hour+":"+minute+":"+second);
            runaControlPanel.setClock("D-"+day+" "+hour+":"+minute+":"+second);
        }
        if(secs!=second0){
            runa.repeatSecondActions(secs-second0);
            second0 = secs;
        }
        if(mins!=minute0){
            runa.repeatMinuteActions(mins-minute0);
            minute0 = mins;
        }
        if(hrs!=hour0){
            runa.repeatHourActions(hrs-hour0);
            hour0 = hrs;
        }
        //System.out.println("currentTime="+currentTime);
    }

    /**
     * Method to change time scale
     * @param ts - new time scale
     */    
    public void setTimeScale(float ts){
        timeScale = ts;
    }
    
    /**
     * Return current time in hours
     * @return - hour and fraction of hour
     */    
    public synchronized static double getCurrentTime(){
        return currentTime;
    }
    
    /**
     * Return current day
     * @return - current day number
     */    
    public synchronized static long getDay(){
        return day;
    }
    
    /**
     * Return current hour
     * @return - current hour number
     */    
    public synchronized static long getHour(){
        return hour;
    }
    
    /**
     * Return current minute
     * @return - current minute number
     */    
    public synchronized static long getMinute(){
        return minute;
    }
    
    /**
     * Return current second
     * @return - current second number
     */    
    public synchronized static long getSecond(){
        return second;
    }
    
    /**
     * Set Runa Control Panel as GUI for Clock object
     * @param rcp - Runa Control Panel
     */    
    public void setRunaControlPanel(RunaControlPanel rcp){
        runaControlPanel = rcp;
    }
    
	/**
	 * @return Returns the clock.
	 */
	public long getClock() {
		return clock;
	}
	/**
	 * @param clock The clock to set.
	 */
	public void setClock(long clock) {
		this.clock = clock;
        second0 = clock/1000 % 60;
        minute0 = second0/60 % 60;
        hour0 = minute0/60 % 24;
	}
	/**
	 * @return Returns the timeScale.
	 */
	public float getTimeScale() {
		return timeScale;
	}
}
