/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * Copyright (C) 2004 Sorokin R.P. (sorokinru@iias.spb.su)
 * and Object-oriented Geoinformatics Systems Laboratory (http://niggis.iias.spb.su)
 * All Rights Reserved.
 */

package ru.igis.omtab;

import edu.stanford.smi.protege.ui.DisplayUtilities;
import edu.stanford.smi.protege.util.CollectionUtilities;
import edu.stanford.smi.protege.widget.*;
import edu.stanford.smi.protege.model.*;

import com.bbn.openmap.*;
import com.bbn.openmap.gui.OverlayMapPanel;
import com.bbn.openmap.util.*;
import java.awt.*;
import java.util.*;

/**
 * Main OpenMap Tab class
 * @author Ru
 * @version 1.0
 */
public class OpenMapTab extends AbstractTabWidget {
    
    /**
	 * Comment for <code>serialVersionUID</code>
	 */
	private static final long serialVersionUID = 1L;
	
	private static OpenMapTab openMapTab;
    private static MapBean mapBean;
    private static OverlayMapPanel overlayMapPanel;
    private static Properties properties;
    private static com.bbn.openmap.gui.MenuBar menuBar;
    
    /** Creates a new instance of OpenMapTab */
    public OpenMapTab() {
        openMapTab = this;
    }
    
    /**
     * Standard Protege method
     */    
    public void setup(WidgetDescriptor descriptor, Project project){
        super.setup(descriptor,project);
        OMTStore.project = project;
        OMTStore.kb = project.getKnowledgeBase();
        OMTStore.runa = null;
        System.gc();
    }
    
    /**
     * Standard Protege method
     */    
	public void initialize() {
        setLabel("OpenMap");
    	Cls omcls = OMTStore.kb.getCls(Ontology.C_OPENMAP);
    	if(omcls != null)
    	{
    		Collection<Instance> ominss = omcls.getInstances();
    		if(!ominss.isEmpty())
    		{
    			Instance omi = findAnnotated(ominss, null);
    			if (omi == null)
    				omi = ominss.iterator().next();
    			setMap(omi);
    		}
    	}
    }
	
	private void setMap(Instance omi){
		Collection<Instance> openmaps = CollectionUtilities.createCollection(omi);
        properties = getKnowledgeBaseProperties(openmaps,new Properties());
        PropertyHandler propertyHandler = new PropertyHandler(properties);
        Debug.init();
        overlayMapPanel = new OverlayMapPanel(propertyHandler);
        mapBean = overlayMapPanel.getMapBean();
        mapBean.showLayerPalettes();
        menuBar = (com.bbn.openmap.gui.MenuBar)overlayMapPanel.getMapMenuBar();
        add(menuBar,BorderLayout.NORTH);
        add((Component)overlayMapPanel,BorderLayout.CENTER); 
	}
    
    @SuppressWarnings("unchecked")
	private Properties getKnowledgeBaseProperties(Collection<Instance> pinstances,Properties properties){
        for(Iterator<Instance> i=pinstances.iterator();i.hasNext();){
            Instance pinstance = i.next();
            Cls directType = pinstance.getDirectType();
            String label = (String)pinstance.getOwnSlotValue(OMTStore.kb.getSlot(Ontology.S_LABEL));
            if (directType.getName().equals(Ontology.C_OPENMAP))
            	label = "openmap";
            Collection<Slot> pslots = directType.getTemplateSlots();
            for(Iterator<Slot> j=pslots.iterator();j.hasNext();){
                Slot pslot = j.next();
                String pslotname = pslot.getName();
                // Discard openmap.label and the same ..
                if(pslotname.equals(Ontology.S_LABEL))
                    continue;
                if(pslotname.startsWith(Ontology.S_PROPERTY)) {
                	String prop_val = (String) pinstance.getOwnSlotValue(pslot);
                	String[] kv = prop_val.split( "=" );
                	if(kv.length == 2)
                		properties.put(label+"."+kv[0], kv[1]);
                	else
                		System.out.println("Wrong slot \"property\" value: "+prop_val);
                    continue;
                }
                String key = label+"."+pslot.getName();
                String value = "";
                if(pslot.getAllowsMultipleValues()){
                    Collection<Instance> values = pinstance.getOwnSlotValues(pslot);
                    value = collectionToString(values);
                    properties = getKnowledgeBaseProperties(values,properties);
                }else
                    value = (String) pinstance.getOwnSlotValue(pslot);
                if(value!=null)
                    properties.put(key, value);
            }
        }
        return properties;
    }
    
    private String collectionToString(Collection<Instance> coll){
        String retval = "";
        for(Iterator<Instance> i=coll.iterator();i.hasNext();){
            Instance inst = i.next();
            retval += (String)inst.getOwnSlotValue(OMTStore.kb.getSlot(Ontology.S_LABEL))+" ";
        }
        return retval;
    }
    
    /**
     * Return stored Protege Knowledge Base
     */    
    public static KnowledgeBase getKB(){
        return OMTStore.kb;
    }
    
    /**
     * Return stored Protege Project
     */    
    public static Project getProj(){
        return OMTStore.project;
    }
    
    /**
     * Return stored RuNA object
     */    
    public static RuNA getRuNA(){
        return OMTStore.runa;
    }
    
    /**
     * Return this object
     */    
    public static OpenMapTab getOpenMapTab(){
        return openMapTab;
    }
	/**
	 * @return Returns the mapBean.
	 */
	public static MapBean getMapBean() {
		return mapBean;
	}
	
    /**
     * Return Instance having annotation (Instance Notes)
     * @param inss - collection of instances to choose from, if null returns first instance with given annotation text
     * @param text - annotation text for search, if null returns first with any annotation
     */    
	public static Instance findAnnotated(Collection<Instance> inss, String text)
	{
		if (inss == null && text == null)
			return null;
		SystemFrames sf = OMTStore.kb.getSystemFrames();
		Cls acls = sf.getAnnotationCls();
		Slot tslot = sf.getAnnotationTextSlot();
		Slot islot = sf.getAnnotatedInstanceSlot();
		Collection<Instance> ainss = acls.getInstances();
		for (Iterator<Instance> iter = ainss.iterator(); iter.hasNext();) {
			Instance ainst = iter.next();
			Object txt = ainst.getOwnSlotValue(tslot);
			Object inst = ainst.getOwnSlotValue(islot);
			if ((inss == null || inss.contains(inst)) && (text == null || text.startsWith((String) txt)))
				return (Instance) inst;
		}
		return null;
	}

    /**
     * Return properties loaded from Knowledge Base during initialization
     */    
	public static Properties getProperties() {
		return properties;
	}
	
	public static void changeMap(){
		if(openMapTab != null){
	        Instance selected = DisplayUtilities.pickInstance(
	                openMapTab,
	                CollectionUtilities.createCollection(OMTStore.kb.getCls(Ontology.C_OPENMAP)),
	                "Select Openmap Instance");
	        if (selected != null) {
	        	openMapTab.remove(menuBar);
	        	openMapTab.remove(overlayMapPanel);
	        	openMapTab.setMap(selected);
	        }
		}
	}
}
