/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * Copyright (C) 2004 Sorokin R.P. (sorokinru@iias.spb.su)
 * and Object-oriented Geoinformatics Systems Laboratory (http://niggis.iias.spb.su)
 * All Rights Reserved.
 */

package ru.igis.omtab;

import ru.igis.omtab.gui.RuMapMouseAdapter;
import ru.igis.omtab.ra.*;

import java.awt.*;
import java.awt.event.MouseEvent;
import java.util.*;

import javax.swing.*;

import com.bbn.openmap.omGraphics.*;
import com.bbn.openmap.plugin.PlugIn;
import com.bbn.openmap.plugin.PlugInLayer;
import com.bbn.openmap.plugin.graphicLoader.GraphicLoaderPlugIn;
import com.bbn.openmap.proj.*;
import com.bbn.openmap.util.*;
import com.bbn.openmap.event.MapMouseListener;
import com.bbn.openmap.graphicLoader.*;

import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.util.*;
import edu.stanford.smi.protege.ui.*;
/**
 * Class supporting simulation
 * @author Ru
 */
@SuppressWarnings("unchecked")
public class RuNA extends MMLGraphicLoader {
    
    private OMGraphicList mapObs = new OMGraphicList();
    private RunaPanel runaPanel;
    private RunaControlPanel runaControlPanel;
    private Clock clock;
    private ArrayList<Instance> kbObs = new ArrayList<Instance>();
    private ArrayList<Instance> raInss = new ArrayList<Instance>();
    private Hashtable<Instance, ProtegeRepeatAction> raHash = new Hashtable<Instance, ProtegeRepeatAction>();
    private Collection<RepeatAction> secondActions = Collections.synchronizedList(new ArrayList<RepeatAction>());
    private Collection<RepeatAction> minuteActions = Collections.synchronizedList(new ArrayList<RepeatAction>());
    private Collection<RepeatAction> hourActions = Collections.synchronizedList(new ArrayList<RepeatAction>());
    private Collection<TaskExecutor> taskExecutors = new ArrayList<TaskExecutor>();
    private String scenario;
    private boolean timerRunning;
    private String timeScale;
    private MultiUser multiUser;
    private OMGraphic selectedGraphic;
    private RuMapMouseAdapter ruMMAdapter;
    private PlugInLayer runaPlugInLayer;

    public PlugInLayer getRuNAPlugInLayer() {
    	if(runaPlugInLayer == null){
    		Component[] comps = OpenMapTab.getMapBean().getComponents();
    		for(int i=0; i<comps.length; i++)
    			if(comps[i] instanceof PlugInLayer) {
    				PlugIn plugin = ((PlugInLayer) comps[i]).getPlugIn();
    				if(plugin != null && plugin instanceof GraphicLoaderPlugIn) {
    					GraphicLoader gloader = ((GraphicLoaderPlugIn) plugin).getGraphicLoader();
    					if(gloader != null && gloader instanceof RuNA) {
    						runaPlugInLayer = (PlugInLayer) comps[i];
    						return runaPlugInLayer;
    					}
    				}
    			}
    		return null;
    	} else
    		return runaPlugInLayer;
	}

	/**
     * Default constructor
     */    
    public RuNA() {
        OMTStore.runa = this;
        clock = new Clock(this);
    }
    
    /**
     * Standard OpenMap method
     */    
    public synchronized void manageGraphics() {
        // 	Debug.message("animationtester", "AnimationActioner.manageGraphics");
    	double currentTime = Clock.getCurrentTime();
        Projection p = getProjection();

        Iterator<?> it = mapObs.iterator();
        while (it.hasNext()) {
            OMGraphic omg = (OMGraphic) it.next();
            if(omg instanceof NavOb)
            	((NavOb)omg).move(currentTime);
            omg.generate(p);
        }

        OMGraphicHandler receiver = getReceiver();
        if (receiver != null) {
            receiver.setList(mapObs);
        }
    }
    
    /**
     * Method that move and display objects on a map periodically
     */    
//     protected synchronized void moveAndDisplayObjects(){
//        double currentTime = Clock.getCurrentTime();
//        Projection p = getProjection();
//        
//        for(Iterator i=mapObs.iterator();i.hasNext();){
//            OMGraphic omg = (OMGraphic) i.next();
//            if(omg instanceof NavOb)
//            {
//                ((NavOb)omg).move(currentTime);
//                omg.generate(p);
//            }
//        }
////        manageGraphics();
//    }

     /**
      * Method called on button Task on Map object form click
      * Calling all registered Task Executors
      * @param no - Navigating Object on wich form button Task was clicked
      */     
    protected synchronized void execTask(NavOb no){
        for(Iterator<TaskExecutor> i=taskExecutors.iterator();i.hasNext();){
            TaskExecutor te = (TaskExecutor)i.next();
            te.execTask(no);
        }
    }
     
    /**
     * Registering new Task Executor
     * @param ta - program called on button Task on NavOb Form click
     */    
    public synchronized void addTaskExecutor(TaskExecutor ta){
        taskExecutors.add(ta);
    }

    /**
     * Remove existing Task Executor
     * @param ta - program called on button Task on NavOb Form click
     */    
    public synchronized void removeTaskExecutor(TaskExecutor ta){
        taskExecutors.remove(ta);
    }

    /**
     * Remove all registered Task Executors atonce
     */    
    public synchronized void clearTaskExecutors(){
        taskExecutors.clear();
    }

    /**
     * Execute all registered Second Actions
     * @param secs - number of elapsed seconds
     */    
    protected synchronized void repeatSecondActions(long secs){
        if (secondActions.size() == 0)
                return;
        ArrayList<RepeatAction> snapshot = new ArrayList<RepeatAction>(secondActions);
        for(Iterator<RepeatAction> i = snapshot.iterator();i.hasNext();){
            RepeatAction ra = (RepeatAction)i.next();
            ra.repeat(secs);
        }
    }
     
    /**
     * Execute all registered Minute Actions
     * @param mins - number of elapsed minutes
     */    
    protected synchronized void repeatMinuteActions(long mins){
        if (minuteActions.size() == 0)
                return;
        ArrayList<RepeatAction> snapshot = new ArrayList<RepeatAction>(minuteActions);
        for(Iterator<RepeatAction> i = snapshot.iterator();i.hasNext();){
            RepeatAction ra = (RepeatAction)i.next();
            ra.repeat(mins);
        }
    }
     
    /**
     * Execute all registered Hour Actions
     * @param hors - number of elapsed hours
     */    
    protected synchronized void repeatHourActions(long hors){
        if (hourActions.size() == 0)
                return;
        ArrayList<RepeatAction> snapshot = new ArrayList<RepeatAction>(hourActions);
        for(Iterator<RepeatAction> i = snapshot.iterator();i.hasNext();){
            RepeatAction ra = (RepeatAction)i.next();
            ra.repeat(hors);
        }
    }
     
    /**
     * Register new Repeat Action for executing every second
     * @param ra - class incapsulating the program called periodically
     */    
    public synchronized void addSecondAction(RepeatAction ra){
        secondActions.add(ra);
    }

    /**
     * Register new Repeat Action for executing every minute
     * @param ra - class incapsulating the program called periodically
     */    
    public synchronized void addMinuteAction(RepeatAction ra){
        minuteActions.add(ra);
    }

    /**
     * Register new Repeat Action for executing every hour
     * @param ra - class incapsulating the program called periodically
     */    
    public synchronized void addHourAction(RepeatAction ra){
        hourActions.add(ra);
    }

    /**
     * Remove existing Repeat Action calling every second
     * @param ra - class incapsulating the program called periodically
     */    
    public synchronized void removeSecondAction(RepeatAction ra){
        secondActions.remove(ra);
    }

    /**
     * Remove existing Repeat Action calling every second
     * @param ra - class incapsulating the program called periodically
     */    
    public synchronized void removeMinuteAction(RepeatAction ra){
        minuteActions.remove(ra);
    }

    /**
     * Remove existing Repeat Action calling every second
     * @param ra - class incapsulating the program called periodically
     */    
    public synchronized void removeHourAction(RepeatAction ra){
        hourActions.remove(ra);
    }

    /**
     * Remove at once all existing Repeat Actions for every second
     */    
    public synchronized void clearSecondActions(){
        secondActions.clear();
    }

    /**
     * Remove at once all existing Repeat Actions for every minute
     */    
    public synchronized void clearMinuteActions(){
        minuteActions.clear();
    }

    /**
     * Remove at once all existing Repeat Actions for every hour
     */    
    public synchronized void clearHourActions(){
        hourActions.clear();
    }

    /**
     * Add Map Obs with picking Instances by user Dialog
     */    
    public void addMapObs() { 
        Collection<?> inss = DisplayUtilities.pickInstances(new JFrame(),OMTStore.kb,Collections.EMPTY_LIST);
        addMapObs(inss);
    }
    
    /**
     * Add Map obs from the collection of Instances
     * @param inss - collection of Instances representing Map Obs in Protege
     */    
    public synchronized void addMapObs(Collection<?> inss) {
        for(Iterator<?> i=inss.iterator();i.hasNext();)
            addMapOb((Instance)i.next());
    }
    
    /**
     * Create from Instance and add to RuNA Map Object
     * @param inst - Intstance representing Map Object in Protege
     * @return Map Object
     */    
    public synchronized MapOb addMapOb(Instance inst) {
        MapOb mapOb = getMapOb(inst);
        try{
            if(mapOb!=null){
            	mapOb.mapFromProtege(inst);
            }else {
                if(inst.hasDirectType(OMTStore.kb.getCls(Ontology.C_MAPOB)))
                    mapOb =  new MapOb(inst);
                else if(inst.hasDirectType(OMTStore.kb.getCls(Ontology.C_NAVOB)))
                    mapOb =  new NavOb(inst); 
                else if(inst.hasDirectType(OMTStore.kb.getCls(Ontology.C_OMPOINT)))
                    mapOb =  new OMTPoint(inst);
                else if(inst.hasDirectType(OMTStore.kb.getCls(Ontology.C_OMRASTER)))
                    mapOb =  new OMTRaster(inst);
                else if(inst.hasDirectType(OMTStore.kb.getCls(Ontology.C_OMPOLY)))
                    mapOb =  new OMTPoly(inst);
                else if(inst.hasDirectType(OMTStore.kb.getCls(Ontology.C_OMSPLINE)))
                    mapOb =  new OMTSpline(inst);
                else if(inst.hasDirectType(OMTStore.kb.getCls(Ontology.C_OMARC)))
                    mapOb =  new OMTArc(inst);
                else if(inst.hasDirectType(OMTStore.kb.getCls(Ontology.C_OMCIRCLE)))
                    mapOb =  new OMTCircle(inst);
                else if(inst.hasDirectType(OMTStore.kb.getCls(Ontology.C_OMRECT)))
                    mapOb =  new OMTRect(inst);
                else if(inst.hasDirectType(OMTStore.kb.getCls(Ontology.C_LINK)))
                    mapOb =  new Link(inst);
                else if(inst.hasDirectType(OMTStore.kb.getCls(Ontology.C_FIELD)))
                    mapOb =  new Field(inst);
                else if(inst.hasDirectType(OMTStore.kb.getCls(Ontology.C_SPILL)))
                    mapOb =  new Spill(inst);
                else if(inst.hasDirectType(OMTStore.kb.getCls(Ontology.C_DRIFT)))
                    mapOb =  new Drift(inst);
                else if(inst.hasDirectType(OMTStore.kb.getCls(Ontology.C_OMTEXT)))
                    mapOb =  new OMTText(inst);
                else
                    System.out.println("OpenMapTab ERROR: Unknown type "+inst.getDirectType());
                if(mapOb!=null){
                    mapObs.add(mapOb);
                    kbObs.add(inst); // accumulate all instances added from kb
                }
            }
        }catch(Exception exc){
            System.out.println("addMapOb "+inst+" Exception: ");
            exc.printStackTrace();
        }
        return mapOb;
    }
    
    /**
     * Delete from RuNA Map Objects corresponding to Instances from Collection.
     * If Collection is nil then delete all Map Objects from RuNA.
     * Simalteniously with Map Objects can delete Instances corresponding them
     * from Protege
     * @param col - Instances corresponding to Map Objects
     * @param kbdelete - should Instances be deleted
     */    
    public synchronized void clearMapObs(Collection<Instance> col,boolean kbdelete) {
        if(col==null){
            if (kbdelete)
                for(Iterator<Instance> i=kbObs.iterator();i.hasNext();)
                    OMTStore.kb.deleteInstance((Instance)i.next());
            mapObs.clear();
            kbObs.clear();
        } else {
        	// First remove links and tows
        	Collection<?> mapObsCopy = new ArrayList<Object>(mapObs.getTargets());
            for(Iterator<?> i=mapObsCopy.iterator();i.hasNext();){
                MapOb mo = (MapOb)i.next();
                Instance inst = mo.getInstance();
                if(col.contains(inst) && !(mo instanceof Link))
                    clearLinksAndTows(mo);
            }
            boolean yet = true;
            while(yet){
                yet = false;
                for(Iterator<?> i=mapObs.iterator();i.hasNext();){
                    MapOb mo = (MapOb)i.next();
                    Instance inst = mo.getInstance();
                    if(col.contains(inst)){
                        i.remove();
                        kbObs.remove(inst);
                        if (kbdelete)
                            OMTStore.kb.deleteInstance(inst);
                        yet = true;
                        break;
                    }
                }
            }
        }
    }

    /**
     * Write back to Protege changed attributies of Map Objects
     * into slots of corresponding Instances
     * @param col - Instances corresponding existing in RuNA Map Objects
     * @throws Exception -
     */    
    public synchronized void mapBackMapObs(Collection<Instance> col) throws Exception {
        if(col==null){
            col = kbObs;
        }
        for(Iterator<?> i=mapObs.iterator();i.hasNext();){
            MapOb mo = (MapOb)i.next();
            Instance inst = mo.getInstance();
            if(col.contains(inst))
                mo.mapToProtege(inst);
        }
    }

    /**
     * Return Runa Panel
     * @return Runa Panel
     */    
    public Component getGUI() {
	if (runaPanel == null) 
            runaPanel = new RunaPanel(this);                                                                 
	return runaPanel;
    }

    /**
     * Handler of button AddObsToMap click
     */    
    public void cmdAddObsToMap(){
        addMapObs();
        if (!getTimer().isRunning()) 
            manageGraphics();
    }
    
    /**
     * Handler of button RemObsFromMap click
     */    
    public void cmdRemObsFromMap(){
        int ans = JOptionPane.showConfirmDialog(new JFrame(),"Remove All Objects?");
        if(ans==JOptionPane.OK_OPTION)
            clearMapObs(null,false);
        else{
            Collection<Instance> col = DisplayUtilities.pickInstancesFromCollection(new JFrame(),kbObs,"Select Objects");
            clearMapObs(col,false);
        }
        if (!getTimer().isRunning()) 
            manageGraphics();
    }
    
    /**
     * Set new time scale
     * @param timeScale - new time scale
     */    
    public void setTimeScale(String timeScale){
        StringTokenizer st = new StringTokenizer(timeScale,":");
        if(st.countTokens() == 2){
        	String nums = st.nextToken();
        	String dens = st.nextToken();
        	try {
				int num = Integer.parseInt(nums);
				int den = Integer.parseInt(dens);
				float ts = (float) num / (float) den;
				if (ts >= 0.001) {
					clock.setTimeScale(ts);
			        this.timeScale = timeScale;
	                if(runaPanel!=null)
	                    runaPanel.updateTimeScaleText(timeScale);
	                if(runaControlPanel!=null)
	                    runaControlPanel.updateTimeScaleText(timeScale);
	                if(multiUser != null && multiUser.isMultiUserMode())
	                	multiUser.toKBTimeScale(timeScale);
				}
			} catch (NumberFormatException e) {}
    	}
    }
	/**
	 * @return Returns the timeScale.
	 */
	public String getTimeScale() {
		return timeScale;
	}
    /**
     * Handler of button Control Object
     */    
    public void cmdControlObject(){
        Collection<?> nobs = Util.filter(kbObs, OMTStore.kb.getCls(Ontology.C_NAVOB));
        if(!nobs.isEmpty()){
            Instance inst = DisplayUtilities.pickInstanceFromCollection(new JFrame(),nobs,0, "Select Object");
            if(inst!=null)
                controlObject(inst);
            else
                return;
        } else
            JOptionPane.showMessageDialog(null,"No objects");
    }
    
    /**
     * Handler of button MapBackObjects
     */    
    public void cmdMapBackObjects(){
        int ans = JOptionPane.showConfirmDialog(new JFrame(),"Map Back All Objects?");
        try{
            if(ans==JOptionPane.OK_OPTION)
                mapBackMapObs(null);
            else{
                Collection<Instance> col = DisplayUtilities.pickInstancesFromCollection(new JFrame(),kbObs,"Select Objects");
                mapBackMapObs(col);
            }
        } catch(Exception exc){
            exc.printStackTrace();
        }
    }
    
    /**
     * Handler of Menu Item Start Repeat Actions click
     */    
    public void cmdStartRepeatActions(){
        Collection<?> racls = OMTStore.kb.getClsNameMatches(Ontology.C_RA,1);
        Collection<?> inss = DisplayUtilities.pickInstances(
            new JFrame(),OMTStore.kb,racls,"Start Repeat Actions"
        );
        startRepeatActions(inss);
   }

    /**
     * Start RepeatActions from the collection of Instances
     * @param inss - collection of Instances of class RepeatAction in Protege
     */    
    public synchronized void startRepeatActions(Collection<?> inss) {
        for(Iterator<?> i=inss.iterator();i.hasNext();)
            startRepeatAction((Instance)i.next());
    }
    
    /**
     * Start RepeatAction from Instance 
     * return false if it is already started or of error
     * @param inst - Intstance of class RepeatAction in Protege
     */    
    public synchronized boolean startRepeatAction(Instance inst) {
        if(raInss.contains(inst))
            return false;
        String clname = "ru.igis.omtab.ra."+inst.getDirectType().getName();
        Class<?> cl = null;
        try{
            cl = Class.forName(clname);
        }catch(ClassNotFoundException e){
            System.out.println("startRepeatAction: Class not found: "+clname);
            return false;
        }
        try{
        	ProtegeRepeatAction pra = (ProtegeRepeatAction)cl.newInstance();
            pra.fillFromProtege(inst,OMTStore.kb,this);
            pra.addIntervalAction(this);
            raInss.add(inst);
            raHash.put(inst,pra);
            return true;
        }catch(Exception e){
            System.out.println("startRepeatAction "+(String)inst.getOwnSlotValue(OMTStore.kb.getSlot(Ontology.S_LABEL))+" Exception: "+e.getMessage());
            return false;
        }
    }
    
    /**
     * Handler of Menu Item Remove Repeat Actions click
     */    
    public void cmdRemoveRepeatActions(){
        Collection<Instance> inss = DisplayUtilities.pickInstancesFromCollection(
            new JFrame(),raInss,"Remove Repeat Actions"
        );
        removeRepeatActions(inss);
    }

    /**
     * Remove RepeatAction from Instance 
     * return false if RA was not started 
     * @param inst - Intstance of class RepeatAction in Protege
     */    
    public synchronized boolean removeRepeatAction(Instance inst) {
        ProtegeRepeatAction pra = (ProtegeRepeatAction)raHash.get(inst);
        if(pra!=null){
            pra.removeIntervalAction(this);
            pra.removeObjects(this);
            raInss.remove(inst);
            return true;
        }
        return false;
    }

    /**
     * Remove RepeatActions from the collection of Instances
     * @param inss - collection of Instances of class RepeatAction in Protege
     */    
    public synchronized void removeRepeatActions(Collection<Instance> inss) {
        for(Iterator<Instance> i=inss.iterator();i.hasNext();)
            removeRepeatAction((Instance)i.next());
    }
    
    /**
     * Search for the Navigating Object corresponding to this Instance
     * and call method control on it
     * @param inst - Instance corresponding to Navigating Object
     */    
    public synchronized void controlObject(Instance inst){
    	NavOb navob = (NavOb) getMapOb(inst);
    	if(navob != null)
    		navob.control();
    }
    
    /**
     * Search Map Object by name
     * @param label - name of Map Object
     * @return Map Object
     */    
    public synchronized MapOb getMapOb(String label){
        for(Iterator<?> i=mapObs.iterator();i.hasNext();){
        	MapOb mo = (MapOb) i.next();
            String name = mo.getName();
            if(name.equals(label))
                return mo;
        }
        return null; 
    }
    
    /**
     * Search polygons containing Map Object
     * @param mo - Map Object
     * @return collection of OMTPoly objects
     */    
    public synchronized Collection<MapOb> inOMTPolies(MapOb mo){
        Collection<MapOb> polies = new ArrayList<MapOb>();
    	for(Iterator<?> i=mapObs.iterator();i.hasNext();){
        	MapOb poly = (MapOb) i.next();
        	if(poly instanceof OMTPoly &&
        		((OMTPoly)poly).contains(mo))
        		polies.add(poly);
        }
        return polies; 
    }
    
    /**
     * Search Map Object for Instance
     * @param instance - Protege Instance corresponding this Map Object
     * @return Map Object
     */    
    public synchronized MapOb getMapOb(Instance instance){
        for(Iterator<?> i=mapObs.iterator();i.hasNext();){
        	MapOb mo = (MapOb) i.next();
            Instance moinst = mo.getInstance();
            if(moinst.equals(instance)){
            	return mo;
            }
        }
        return null; 
    }
    
    /**
     * Gives an array of Map Objects for the array of their types.
     * If array of types is empty return all Map Objects
     * @param typs - array of Map Object names
     * @return Array of Map Objects
     */    
    @SuppressWarnings("rawtypes")
	public synchronized Object[] getNavObs(String[] typs){
        java.util.List types = Arrays.asList(typs);
        ArrayList<NavOb> navobs = new ArrayList<NavOb>();
        for(Iterator<?> i=mapObs.iterator();i.hasNext();){
	    MapOb mo = (MapOb) i.next();
            if(mo instanceof NavOb){
                NavOb no = (NavOb)mo;
                if(types.isEmpty() || types.contains(no.getType()))
                    navobs.add(no);
            }
        }
        return navobs.toArray(); 
    }
    
    /**
     * Gives an array of mooving Map Objects for the array of their names
     * @param typs - array of Map Object names
     * @return Array of Map Objects
     */    
    @SuppressWarnings("rawtypes")
	public synchronized Object[] getMovingNavObs(String[] typs){
        java.util.List types = Arrays.asList(typs);
        ArrayList<NavOb> navobs = new ArrayList<NavOb>();
        for(Iterator<?> i=mapObs.iterator();i.hasNext();){
	    MapOb mo = (MapOb) i.next();
            if(mo instanceof NavOb){
                NavOb no = (NavOb)mo;
                if(no.getSpeed()>0f && (types.isEmpty() || types.contains(no.getType())))
                    navobs.add(no);
            }
        }
        return navobs.toArray(); 
    }
    
    /**
     * Add to RuNA new Map Object.
     * If Map Object with the same name exists then it is removed first
     * @param mo - new Map Object
     */    
    public synchronized void addMapOb(MapOb mo){
        MapOb old = getMapOb(mo.getName());
        if(old!=null)
            removeMapOb(old,false);
        mapObs.add(mo);
        kbObs.add(mo.getInstance());
    }
    
    /**
     * Remove from RuNA existing Map Object
     * @param mo - Map Object to remove
     * @param kbdelete Should corresponding Protege Instance be removed too
     */    
    public synchronized void removeMapOb(MapOb mo,boolean kbdelete){
        if(!(mo instanceof Link))
        	clearLinksAndTows(mo);
    	mapObs.remove(mo);
        Instance inst = mo.getInstance();
        kbObs.remove(inst);
        if(kbdelete)
            OMTStore.kb.deleteInstance(inst);
    }
    
    private void clearLinksAndTows(MapOb mo)
    {
        mo.offTow();
        if(mo instanceof NavOb)
        {
        	((NavOb)mo).clearTows();
        	Collection<?> mapObLinks = ((NavOb)mo).getLinks();
            if(mapObLinks != null)
            	for (Iterator<?> iter = mapObLinks.iterator(); iter.hasNext();) 
    				removeMapOb((MapOb) iter.next(), true);
        }
        Collection<?> linksToMapOb = mo.getLinksToMe();
        if(linksToMapOb != null)
        	for (Iterator<?> iter = linksToMapOb.iterator(); iter.hasNext();) 
				removeMapOb((MapOb) iter.next(), true);
    }

    /**
     * Standard OpenMap Component method
     * @param prefix - propery prefix
     * @param props - OpenMap properties object
     */    
    public void setProperties (String prefix, Properties props) {
        super.setProperties(prefix, props);
        String realPrefix = PropUtils.getScopedPropertyPrefix(prefix);
        scenario = props.getProperty(realPrefix + Ontology.S_SCENARIO);
        System.out.println("OpenMapTab scenario = "+scenario);
    }
    
    /**
     * Method that return contains of scenario slot of the first instance
     * of class RuNA in Protege
     * @return - contains of scenario slot
     */    
    public String getScenario(){
        // To make online scenario change
        Cls runacls = OMTStore.kb.getCls(Ontology.C_RUNA);
        if(runacls!=null){
            Collection<?> coll = runacls.getInstances();
            if(coll.size()>0){
                Instance inst = (Instance)coll.iterator().next();
                scenario = (String)inst.getOwnSlotValue(OMTStore.kb.getSlot(Ontology.S_SCENARIO));
            }
        }
        return scenario;
    }
    
    /**
     * Return contains of slot scenario of picked by user Instanse
     * of class Command or its subclass that is a value of slot
     * commandSubclass of first instance of class RuNA in Protege
     * @return - command String
     */    
    public String getCommand(){
        // To make online scenario change
        Cls runacls = OMTStore.kb.getCls(Ontology.C_RUNA);
        if(runacls!=null){
            Collection<?> coll = runacls.getInstances();
            if(coll.size()>0){
                Instance inst = (Instance)coll.iterator().next();
                String scomcls = (String)inst.getOwnSlotValue(OMTStore.kb.getSlot(Ontology.S_COMSUBCLS));
                Cls comcls;
                if(scomcls!=null)
                    comcls = OMTStore.kb.getCls(scomcls);
                else
                    comcls = OMTStore.kb.getCls(Ontology.C_COMMAND);
                Instance cominst = DisplayUtilities.pickInstance(
                    runaControlPanel, 
                    CollectionUtilities.createCollection(comcls)
                );
                if(cominst!=null){
                    return (String)cominst.getOwnSlotValue(OMTStore.kb.getSlot(Ontology.S_SCENARIO));
                }
            }
        }
        return null;
    }
    
    /**
     * Return Clock object
     * @return - Clock object
     */    
    public Clock getClock(){
        return clock;
    }
    
    /**
     * Standard OpenMap interface LightMapHandlerChild method.
     * Search for RunaControlPanel and set it for this RuNA object
     * and also send it to Clock object
     * @param obj - serched object
     */    
    public void findAndInit(Object obj) {
        if (obj instanceof RunaControlPanel) {
            runaControlPanel = (RunaControlPanel)obj;
            clock.setRunaControlPanel(runaControlPanel);
        } else if (obj instanceof RuMapMouseAdapter) {
        	setRuMapMouseAdapter((RuMapMouseAdapter) obj);
        }
    }

    /**
     * Standard OpenMap interface LightMapHandlerChild method.
     * Set to null links to RunaControlPanel in this object and
     * in Clock object
     * @param obj - searched object
     */    
    public void findAndUndo(Object obj) {
        if (obj instanceof RunaControlPanel) {
            runaControlPanel = null;
            clock.setRunaControlPanel(null);
        } else if (obj instanceof RuMapMouseAdapter) {
        	ruMMAdapter = null;
        }
    }
    public void setReflectNavOb(boolean flag){
    	OMTStore.reflectNavOb = flag;
    }
	/**
	 * @return Returns the mapObs.
	 */
	public OMGraphicList getMapObs() {
		return mapObs;
	}
	/**
	 * @return Returns the timerRunning.
	 */
	public boolean isTimerRunning() {
		return timerRunning;
	}
	/**
	 * @param timerRunning The timerRunning to set.
	 */
	public void setTimerRunning(boolean timerRunning) {
		setTimerRunning(timerRunning, true);
	}
	protected void setTimerRunning(boolean timerRunning, boolean updateGUI) {
		this.timerRunning = timerRunning;
        if (timerRunning) {
            manageGraphics();
            if(updateGUI){
                if(runaPanel!=null)
                    runaPanel.setRunTimerSelected(true);
                if(runaControlPanel!=null)
                    runaControlPanel.setRunTimerSelected(true);
            }
            if(runaPanel!=null)
                runaPanel.setSliderEnabled(false); // to prevent moving objects during slider adjustment
            if(runaControlPanel!=null)
                runaControlPanel.setSliderEnabled(false);  // to prevent moving objects during slider adjustment
			getTimer().restart();
        } else {
            getTimer().stop();
            if(updateGUI){
                if(runaPanel!=null)
                    runaPanel.setRunTimerSelected(false);
                if(runaControlPanel!=null)
                    runaControlPanel.setRunTimerSelected(false);
            }
            if(runaPanel!=null)
                runaPanel.setSliderEnabled(true); // release slider adjustment
            if(runaControlPanel!=null)
                runaControlPanel.setSliderEnabled(true);  // release slider adjustment
        }
        if(multiUser != null && multiUser.isMultiUserMode() && multiUser.isLeader())
        	multiUser.toKBRunning(timerRunning);
	}
	/**
	 * @return Returns the multiUser.
	 */
	public MultiUser getMultiUser() {
		return multiUser;
	}
	/**
	 * @param multiUser The multiUser to set.
	 */
	public void setMultiUser(MultiUser multiUser) {
		this.multiUser = multiUser;
	}
	
	protected void disableDrivenGUI(){
        if(runaPanel!=null)
            runaPanel.disableDrivenGUI();
        if(runaControlPanel!=null)
            runaControlPanel.disableDrivenGUI();
	}
	/**
	 * @return Returns collection of instances corresponding to list of mapObs.
	 */
	public Collection<Instance> getKbObs() {
		return kbObs;
	}
	
	public MapMouseListener getMapMouseListener() {
		return this;
	}

    /**
     * Called whenever the mouse is moved on this layer and one of the
     * requested mouse modes is active.
     * <p>
     * Tries to locate a graphic near the mouse, and if it is found,
     * it selected.
     * 
     * @param e the move event
     * @return true if event was consumed (handled), false otherwise
     * @see #getMouseModeServiceList
     */
    public boolean mouseMoved(MouseEvent e) {
        OMGraphic newSelectedGraphic = mapObs.selectClosest(e.getX(),
                e.getY(),
                2.0f);
        if (newSelectedGraphic != selectedGraphic) {
            selectedGraphic = newSelectedGraphic;
            if(ruMMAdapter != null)
            	ruMMAdapter.mouseOn((MapOb) selectedGraphic, getRuNAPlugInLayer());
        }
        return true;
    }

    /**
     * Called whenever the mouse is moved on this layer and one of the
     * requested mouse modes is active, and the gesture is consumed by
     * another active layer. We need to deselect anything that may be
     * selected.
     * 
     * @see #getMouseModeServiceList
     */
    public void mouseMoved() {
    	mapObs.deselect();
    }

    /**
     * Called whenever the mouse is clicked by the user and one of the
     * requested mouse modes is active.
     * 
     * @param e the click event
     * @return true if event was consumed (handled), false otherwise
     * @see #getMouseModeServiceList
     */
    public boolean mouseClicked(MouseEvent e) {
        if (ruMMAdapter != null) {
        	if(e.getButton() == 1)
        		ruMMAdapter.mouseButton1ClickedOn((MapOb) selectedGraphic, getRuNAPlugInLayer());
        	else if(e.getButton() == 3)
        		ruMMAdapter.mouseButton3ClickedOn((MapOb) selectedGraphic, getRuNAPlugInLayer());
        	else if(e.getButton() == 2)
        		ruMMAdapter.mouseButton2ClickedOn((MapOb) selectedGraphic, getRuNAPlugInLayer());
        	else
        		return false;
            return true;
        } else {
            return false;
        }
    }

	public RuMapMouseAdapter getRuMapMouseAdapter() {
		return ruMMAdapter;
	}

	public void setRuMapMouseAdapter(RuMapMouseAdapter ruMMAdapter) {
		this.ruMMAdapter = ruMMAdapter;
	}
    
}
